/** ZCT/cmd.c--Created 060205 **/
#include "zct.h"
#include "functions.h"
#include "globals.h"
#include "cmd.h"

/* Prototypes for this file */
COMMAND *cmd_lookup(char *input);
CMD_HASHKEY cmd_hashkey(char *input);

#ifndef ZCT_POSIX
char *strsep(char **stringp, const char *delim);
#endif

/* Command-globals */
CMD_INPUT cmd_input;
PROTOCOL last_protocol;
COMMAND tune_commands[] =
{
	{ 0, "exit", "Exit the tuning process.", 1, cmd_null },
	{ 0, NULL, NULL, 0, NULL }
};
/* Locals */
COMMAND *commands[] = { def_commands, xb_commands, uci_commands, an_commands, an_commands, dbg_commands, tune_commands };
COMMAND cmd_table[CMD_TABLE_SIZE];

/**
command():
Processes user input.
Created 060205; last modified 020808
**/
int command(char *input)
{
	COMMAND *cmd;

	if (input[0] == '!')
	{
		system(input + 1);
		return 1;
	}
	strcpy(cmd_input.input, input);
	cmd_parse(" \t\n");
	/* No arguments. */
	if (cmd_input.arg_count == 0)
		return 0;
	/* Comment: mainly used for .ini files. */
	if (cmd_input.arg[0][0] == '#')
		return 1;
	cmd = cmd_lookup(cmd_input.arg[0]);
	if (cmd)
	{
		if ((cmd->search_flag == 1 && engine_state != IDLE) || (cmd->search_flag == 2 && engine_state == PONDERING))
			return 2;
		cmd->cmd_func();
		return 1;
	}
	return 0;
}

/**
initialize_cmds():
Initializes the internal command tables.
Created 070105; last modified 102506
**/
void initialize_cmds(void)
{
	int x;
	CMD_HASHKEY key;

	for (x = 0; x < CMD_TABLE_SIZE; x++)
		cmd_table[x].cmd_name = NULL;
	for (x = 0; commands[protocol][x].cmd_name != NULL; x++)
	{
		for (key = cmd_hashkey(commands[protocol][x].cmd_name); key < CMD_TABLE_SIZE && cmd_table[key].cmd_name != NULL; key++)
			;
		if (key < CMD_TABLE_SIZE)
		{
			cmd_table[key] = commands[protocol][x];
			cmd_table[key].hashkey = key;
		}
	}
}

/**
cmd_parse():
Parses an input string into arguments.
Created 081205; last modified 110506
**/
void cmd_parse(char *delim)
{
	char *next;
	char *str;

	strcpy(cmd_input.args, cmd_input.input);
	str = cmd_input.args;
	for (cmd_input.arg_count = 0; (next = strsep(&str, delim)) != NULL; cmd_input.arg_count++)
	{
		if (*next != '\0')
		{
			cmd_input.arg[cmd_input.arg_count] = next;
			/* Check if the next argument is enclosed in quotes. */
			if (str != NULL && *str == '"')
			{
				str++; /* Advance past the quote character */
				cmd_input.arg_count++;
				cmd_input.arg[cmd_input.arg_count] = str;
				if ((str = strchr(str, '"')) != NULL)
					*str++ = '\0'; /* Delete the ending quote character */
			}	
		}
		else
			cmd_input.arg_count--;
	}
}

/**
display_help():
Displays a list of commands in the current protocol for the user.
Created 071207; last modified 071207
**/
void display_help(void)
{
	COMMAND *cmd;

	for (cmd = commands[protocol]; cmd->cmd_func != NULL; cmd++)
		if (cmd->help_text != NULL)
			print("%-10s    %s\n", cmd->cmd_name, cmd->help_text);
}

/**
cmd_lookup():
Look up command on internal hash table.
Created 070105; last modified 102506
**/
COMMAND *cmd_lookup(char *input)
{
	CMD_HASHKEY key;
	for (key = cmd_hashkey(input); key < CMD_TABLE_SIZE && cmd_table[key].cmd_name != NULL; key++)
		if (!strcmp(input, cmd_table[key].cmd_name))
			return &cmd_table[key];
	return NULL;
}

/**
cmd_hashkey():
Creates a hash key from input for the command lookup table.
Created 060205; last modified 123107
**/
CMD_HASHKEY cmd_hashkey(char *input)
{
	int h;
	h = strlen(input) * 319;
	h += input[0];
	h *= input[strlen(input) >> 1];
	h -= input[strlen(input) - 1];
	return h % CMD_TABLE_SIZE;
}


#if !defined(ZCT_POSIX)

char *strsep(char **stringp, const char *delim)
{
    register char *s;
    register const char *spanp;
    register int c, sc;
    char *tok;

    if ((s = *stringp) == NULL)
    	return (NULL);
	for (tok = s;;) {
    	c = *s++;
        spanp = delim;
        do {
        	if ((sc = *spanp++) == c) {
            	if (c == 0)
                    s = NULL;
                else
                    s[-1] = 0;
                *stringp = s;
                return (tok);
         	}
        } while (sc != 0);
    }
}
#endif

